@@ -5,14 +5,16 @@ use warnings;
 package Dist::Zilla::Plugin::InstallGuide;
 
 # ABSTRACT: Build an INSTALL file
-our $VERSION = '1.200003'; # VERSION
+our $VERSION = '1.200005'; # VERSION
 use Moose;
 use Moose::Autobox;
 with 'Dist::Zilla::Role::FileGatherer';
 with 'Dist::Zilla::Role::TextTemplate';
+with 'Dist::Zilla::Role::FileMunger';
+use List::Util 'first';
 
 
-my $template = q|
+has template => (is => 'ro', isa => 'Str', default => <<'END_TEXT');
 This is the Perl distribution {{ $dist->name }}.
 
 Installing {{ $dist->name }} is straightforward.
@@ -35,6 +37,7 @@ Alternatively, if your CPAN shell is set up, you should just be able to do:
     % cpan {{ $package }}
 
 ## Manual installation
+
 {{ $manual_installation }}
 ## Documentation
 
@@ -42,9 +45,9 @@ Alternatively, if your CPAN shell is set up, you should just be able to do:
 You can run perldoc from a shell to read the documentation:
 
     % perldoc {{ $package }}
-|;
+END_TEXT
 
-my $makemaker_manual_installation = q|
+has makemaker_manual_installation => (is => 'ro', isa => 'Str', default => <<'END_TEXT');
 As a last resort, you can manually install it. Download the tarball, untar it,
 then build it:
 
@@ -58,9 +61,9 @@ Then install it:
 If you are installing into a system-wide directory, you may need to run:
 
     % sudo make install
-|;
+END_TEXT
 
-my $module_build_manual_installation = q|
+has module_build_manual_installation => (is => 'ro', isa => 'Str', default => <<'END_TEXT');
 As a last resort, you can manually install it. Download the tarball, untar it,
 then build it:
 
@@ -74,49 +77,62 @@ Then install it:
 If you are installing into a system-wide directory, you may need to run:
 
     % sudo ./Build install
-|;
+END_TEXT
 
 
 sub gather_files {
     my $self = shift;
 
-    require Dist::Zilla::File::FromCode;
-
-    my $zilla = $self->zilla;
-    $self->add_file(Dist::Zilla::File::FromCode->new({
+    require Dist::Zilla::File::InMemory;
+    $self->add_file(Dist::Zilla::File::InMemory->new({
         name => 'INSTALL',
-        code => sub {
-            my $manual_installation = '';
-
-            my %installer = map { $_->name => 1 }
-                grep { $_->name eq 'Makefile.PL' or $_->name eq 'Build.PL' }
-                @{ $zilla->files };
-
-            if ($installer{'Build.PL'}) {
-                $manual_installation .= $module_build_manual_installation;
-            }
-            elsif ($installer{'Makefile.PL'}) {
-                $manual_installation .= $makemaker_manual_installation;
-            }
-            unless ($manual_installation) {
-                $self->log_fatal('neither Makefile.PL nor Build.PL is present, aborting');
-            }
-
-            (my $main_package = $zilla->name) =~ s!-!::!g;
-
-            my $content = $self->fill_in_string(
-                $template,
-                {   dist                => \$zilla,
-                    package             => $main_package,
-                    manual_installation => $manual_installation
-                }
-            );
-            return $content;
-        },
+        content => $self->template,
     }));
 
     return;
 }
+
+
+sub munge_files {
+    my $self = shift;
+
+    my $zilla = $self->zilla;
+
+    my $manual_installation = '';
+
+    my %installer = (
+        map {
+            $_->isa('Dist::Zilla::Plugin::MakeMaker') ? ( 'Makefile.PL' => 1 ) : (),
+            $_->does('Dist::Zilla::Role::BuildPL') ? ( 'Build.PL' => 1 ) : (),
+        } @{ $zilla->plugins }
+    );
+
+    if ($installer{'Build.PL'}) {
+        $manual_installation .= $self->module_build_manual_installation;
+    }
+    elsif ($installer{'Makefile.PL'}) {
+        $manual_installation .= $self->makemaker_manual_installation;
+    }
+    unless ($manual_installation) {
+        $self->log_fatal('neither Makefile.PL nor Build.PL is present, aborting');
+    }
+
+    (my $main_package = $zilla->name) =~ s!-!::!g;
+
+    my $file = first { $_->name eq 'INSTALL' } @{ $zilla->files };
+
+    my $content = $self->fill_in_string(
+        $file->content,
+        {   dist                => \$zilla,
+            package             => $main_package,
+            manual_installation => $manual_installation
+        }
+    );
+
+    $file->content($content);
+    return;
+}
+
 __PACKAGE__->meta->make_immutable;
 no Moose;
 1;
@@ -133,7 +149,7 @@ Dist::Zilla::Plugin::InstallGuide - Build an INSTALL file
 
 =head1 VERSION
 
-version 1.200003
+version 1.200005
 
 =head1 SYNOPSIS
 
@@ -155,8 +171,11 @@ appropriate.
 
 =head2 gather_files
 
-Creates the C<INSTALL> file and prepare its contents, which will be finalized
-near the end of the build process.
+Creates the F<INSTALL> file.
+
+=head2 munge_files
+
+Inserts the appropriate installation instructions into F<INSTALL>.
 
 =for test_synopsis 1;
 __END__
